import os
from ckan.common import config
from ckan.lib.navl.dictization_functions import Missing
import ckan.logic as logic
from mock import patch

import ckanext.odsh.validation as validation


def resource_dir():
    this_module_dir = os.path.dirname(os.path.abspath(__file__))
    result = os.path.abspath(os.path.join(this_module_dir, 'resources/'))
    print(result)
    return result


def license_file_url():
    result = 'file://' + \
        os.path.abspath(os.path.join(resource_dir(), 'licenses.json'))
    return result


class WithConfig(object):
    def setUp(self):
        config.update({
            'ckanext.odsh.require_license_attribution': 'True',
            'licenses_group_url': license_file_url(),
        })


class WithFrontendValidationMocks(WithConfig):
    def setUp(self):
        super(WithFrontendValidationMocks, self).setUp()

        self.key_mock = ('extras', 0, 'key')

        self.data_mock_without_groups = {
            ('__extras',): {
                'pkg_name': u'458ccd2c-5e1b-474f-943f-3bab9439a0e5',
                'spatial_uri_temp': u'',
                'tags': []
            },
            ('extras', 0, 'key'): u'groups',
            ('extras', 0, 'revision_timestamp'): Missing(),
            ('extras', 0, 'state'): Missing(),
            ('extras', 0, 'value'): u'',
            ('extras', 1, 'deleted'): Missing(),
            ('extras', 1, 'id'): Missing(),
            ('extras', 1, 'key'): u'issued',
            ('extras', 1, 'revision_timestamp'): Missing(),
            ('extras', 1, 'state'): Missing(),
            ('extras', 1, 'value'): u'2019-12-17',
            ('extras', 2, 'deleted'): Missing(),
            ('extras', 2, 'id'): Missing(),
            ('extras', 2, 'key'): u'temporal_end',
            ('extras', 2, 'revision_timestamp'): Missing(),
            ('extras', 2, 'state'): Missing(),
            ('extras', 2, 'value'): u'',
            ('extras', 3, 'deleted'): Missing(),
            ('extras', 3, 'id'): Missing(),
            ('extras', 3, 'key'): u'temporal_start',
            ('extras', 3, 'revision_timestamp'): Missing(),
            ('extras', 3, 'state'): Missing(),
            ('extras', 3, 'value'): u'',
            ('extras', 4, 'deleted'): Missing(),
            ('extras', 4, 'id'): Missing(),
            ('extras', 4, 'key'): u'licenseAttributionByText',
            ('extras', 4, 'revision_timestamp'): Missing(),
            ('extras', 4, 'state'): Missing(),
            ('extras', 4, 'value'): u'Test-Organisation',
            ('extras', 5, 'deleted'): Missing(),
            ('extras', 5, 'id'): Missing(),
            ('extras', 5, 'key'): 'thumbnail',
            ('extras', 5, 'revision_timestamp'): Missing(),
            ('extras', 5, 'state'): Missing(),
            ('extras', 5, 'value'): u'thumbnail_picture_4519ffc6a956a638b7b6f7a6a82418.jpg',
            ('extras', 6, 'key'): 'language',
            ('extras', 6, 'value'): u'http://publications.europa.eu/resource/authority/language/DEU',
            ('extras', 7, 'key'): 'subject_text',
            ('extras', 7, 'value'): u'Verwaltungsvorschrift',
            ('extras', 8, 'key'): 'subject',
            ('extras', 8, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Verwaltungsvorschrift',
            ('extras', 9, 'key'): 'spatial_uri',
            ('extras', 9, 'value'): u'',
            ('id',): u'458ccd2c-5e1b-474f-943f-3bab9439a0e5',
            ('language',): u'http://publications.europa.eu/resource/authority/language/DEU',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-by-de/2.0',
            ('notes',): u'',
            ('owner_org',): u'63c87e74-60a9-4a4a-a980-d7983c47f92b',
            ('private',): False,
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Verwaltungsvorschrift',
            ('tag_string',): u'',
            ('title',): u'Link again',
            ('type',): u'dataset'
        }

        self.data_mock_with_groups = {
            ('__extras',): {'pkg_name': u'458ccd2c-5e1b-474f-943f-3bab9439a0e5',
                            'spatial_uri_temp': u'',
                            'tags': []},
            ('extras', 0, 'key'): u'groups',
            ('extras', 0, 'value'): u'soci,ener,intr',
            ('extras', 1, 'key'): u'issued',
            ('extras', 1, 'value'): u'2019-12-17',
            ('extras', 2, 'key'): u'temporal_end',
            ('extras', 2, 'value'): u'',
            ('extras', 3, 'key'): u'temporal_start',
            ('extras', 3, 'value'): u'',
            ('extras', 5, 'key'): 'thumbnail',
            ('extras', 5, 'value'): u'thumbnail_picture_4519ffc6a956a638b7b6f7a6a82418.jpg',
            ('extras', 6, 'key'): 'language',
            ('extras', 6, 'value'): u'http://publications.europa.eu/resource/authority/language/DEU',
            ('extras', 7, 'key'): 'subject_text',
            ('extras', 7, 'value'): u'Verwaltungsvorschrift',
            ('extras', 8, 'key'): 'subject',
            ('extras', 8, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Verwaltungsvorschrift',
            ('extras', 9, 'key'): 'spatial_uri',
            ('extras', 9, 'value'): u'',
            ('extras', 10, 'key'): 'licenseAttributionByText',
            ('extras', 10, 'value'): '',
            ('extras', 11, 'key'): 'licenseAttributionByText',
            ('extras', 11, 'value'): '',
            ('extras', 12, 'key'): 'licenseAttributionByText',
            ('extras', 12, 'value'): '',
            ('extras', 13, 'key'): 'licenseAttributionByText',
            ('extras', 13, 'value'): '',
            ('extras', 14, 'key'): 'licenseAttributionByText',
            ('extras', 14, 'value'): '',
            ('extras', 15, 'key'): 'licenseAttributionByText',
            ('extras', 15, 'value'): '',
            ('id',): u'458ccd2c-5e1b-474f-943f-3bab9439a0e5',
            ('language',): u'http://publications.europa.eu/resource/authority/language/DEU',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('notes',): u'',
            ('owner_org',): u'63c87e74-60a9-4a4a-a980-d7983c47f92b',
            ('private',): False,
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Verwaltungsvorschrift',
            ('tag_string',): u'',
            ('title',): u'Link again',
            ('type',): u'dataset'
        }

        self.error_mock = {
            ('__before',): [],
            ('__extras',): [],
            ('__junk',): [],
            ('author',): [],
            ('author_email',): [],
            ('extras', 0, '__extras'): [],
            ('extras', 0, 'deleted'): [],
            ('extras', 0, 'id'): [],
            ('extras', 0, 'key'): [],
            ('extras', 0, 'revision_timestamp'): [],
            ('extras', 0, 'state'): [],
            ('extras', 0, 'value'): [],
            ('extras', 1, '__extras'): [],
            ('extras', 1, 'deleted'): [],
            ('extras', 1, 'id'): [],
            ('extras', 1, 'key'): [],
            ('extras', 1, 'revision_timestamp'): [],
            ('extras', 1, 'state'): [],
            ('extras', 1, 'value'): [],
            ('extras', 2, '__extras'): [],
            ('extras', 2, 'deleted'): [],
            ('extras', 2, 'id'): [],
            ('extras', 2, 'key'): [],
            ('extras', 2, 'revision_timestamp'): [],
            ('extras', 2, 'state'): [],
            ('extras', 2, 'value'): [],
            ('extras', 3, '__extras'): [],
            ('extras', 3, 'deleted'): [],
            ('extras', 3, 'id'): [],
            ('extras', 3, 'key'): [],
            ('extras', 3, 'revision_timestamp'): [],
            ('extras', 3, 'state'): [],
            ('extras', 3, 'value'): [],
            ('extras', 4, '__extras'): [],
            ('extras', 4, 'deleted'): [],
            ('extras', 4, 'id'): [],
            ('extras', 4, 'key'): [],
            ('extras', 4, 'revision_timestamp'): [],
            ('extras', 4, 'state'): [],
            ('extras', 4, 'value'): [],
            ('extras', 5, '__extras'): [],
            ('extras', 5, 'deleted'): [],
            ('extras', 5, 'id'): [],
            ('extras', 5, 'key'): [],
            ('extras', 5, 'revision_timestamp'): [],
            ('extras', 5, 'state'): [],
            ('extras', 5, 'value'): [],
            ('id',): [],
            ('language',): [],
            ('license_id',): [],
            ('log_message',): [],
            ('maintainer',): [],
            ('maintainer_email',): [],
            ('name',): [],
            ('notes',): [],
            ('owner_org',): [],
            ('private',): [],
            ('return_to',): [],
            ('revision_id',): [],
            ('save',): [],
            ('state',): [],
            ('subject',): [],
            ('tag_string',): [],
            ('thumbnail',): [],
            ('title',): [],
            ('type',): [],
            ('url',): [],
            ('version',): []
        }

        self.context_mock = None


class WithAPIValidationMocks(WithConfig):
    def setUp(self):
        super(WithAPIValidationMocks, self).setUp()

        self.data_mock_with_groups = {
            ('__extras',): {'tags': []},
            ('extras', 0, 'key'): u'issued',
            ('extras', 0, 'value'): u'2020-02-28T00:00:00.000',
            ('extras', 1, 'key'): 'subject_text',
            ('extras', 1, 'value'): u'Gutachten',
            ('extras', 2, 'key'): 'subject',
            ('extras', 2, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('extras', 3, 'key'): 'licenseAttributionByText',
            ('extras', 3, 'value'): '',
            ('groups', 0, 'name'): u'heal',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('name',): u'aaa',
            ('owner_org',): u'ce91bacd-043d-4546-8239-c891d4a221f8',
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('title',): u'aaa',
            ('type',): u'dataset'
        }

        self.data_mock_with_empty_groups = {
            ('__extras',): {'groups': [], 'tags': []},
            ('extras', 0, 'key'): u'issued',
            ('extras', 0, 'value'): u'2020-02-28T00:00:00.000',
            ('extras', 1, 'key'): 'subject_text',
            ('extras', 1, 'value'): u'Gutachten',
            ('extras', 2, 'key'): 'subject',
            ('extras', 2, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('extras', 3, 'key'): 'licenseAttributionByText',
            ('extras', 3, 'value'): '',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('name',): u'aaa',
            ('owner_org',): u'ce91bacd-043d-4546-8239-c891d4a221f8',
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('title',): u'aaa',
            ('type',): u'dataset'
        }

        self.data_mock_with_groups_in_extras = {
            ('__extras',): {'tags': []},
            ('extras', 0, 'key'): u'issued',
            ('extras', 0, 'value'): u'2020-02-28T00:00:00.000',
            ('extras', 1, 'key'): u'groups',
            ('extras', 1, 'value'): u'soci,ener,intr',
            ('extras', 2, 'key'): 'subject_text',
            ('extras', 2, 'value'): u'Gutachten',
            ('extras', 3, 'key'): 'subject',
            ('extras', 3, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('extras', 4, 'key'): 'licenseAttributionByText',
            ('extras', 4, 'value'): '',
            ('extras', 5, 'key'): 'licenseAttributionByText',
            ('extras', 5, 'value'): '',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('name',): u'aaa',
            ('owner_org',): u'ce91bacd-043d-4546-8239-c891d4a221f8',
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('title',): u'aaa',
            ('type',): u'dataset'
        }

        self.data_mock_with_empty_groups_in_extras = {
            ('__extras',): {'tags': []},
            ('extras', 0, 'key'): u'issued',
            ('extras', 0, 'value'): u'2020-02-28T00:00:00.000',
            ('extras', 1, 'key'): u'groups',
            ('extras', 1, 'value'): u'',
            ('extras', 2, 'key'): 'subject_text',
            ('extras', 2, 'value'): u'Gutachten',
            ('extras', 3, 'key'): 'subject',
            ('extras', 3, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('extras', 4, 'key'): 'licenseAttributionByText',
            ('extras', 4, 'value'): '',
            ('extras', 5, 'key'): 'licenseAttributionByText',
            ('extras', 5, 'value'): '',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('name',): u'aaa',
            ('owner_org',): u'ce91bacd-043d-4546-8239-c891d4a221f8',
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('title',): u'aaa',
            ('type',): u'dataset'
        }

        self.data_mock_without_groups = {
            ('__extras',): {'tags': []},
            ('extras', 0, 'key'): u'issued',
            ('extras', 0, 'value'): u'2020-02-28T00:00:00.000',
            ('extras', 1, 'key'): 'subject_text',
            ('extras', 1, 'value'): u'Gutachten',
            ('extras', 2, 'key'): 'subject',
            ('extras', 2, 'value'): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('extras', 3, 'key'): 'licenseAttributionByText',
            ('extras', 3, 'value'): '',
            ('license_id',): u'http://dcat-ap.de/def/licenses/dl-zero-de/2.0',
            ('name',): u'aaa',
            ('owner_org',): u'ce91bacd-043d-4546-8239-c891d4a221f8',
            ('subject',): u'http://transparenz.schleswig-holstein.de/informationsgegenstand#Gutachten',
            ('title',): u'aaa',
            ('type',): u'dataset'
        }

        self.errors_mock = {}


patch_get_dataset = patch.object(
    validation,
    'get_package_dict',
)


patch_get_dataset_not_found = patch.object(
    validation,
    'get_package_dict',
    side_effect=logic.NotFound,
)